==> ip2location-iso3166-2-0.2.0.crate: ip2location-iso3166-2-0.2.0/src/csv_format.rs <==
use core::{fmt, ops::Deref};
use std::io::Read;

use csv::{Error as CsvError, Reader as CsvReader};

use crate::record::Record;

//
#[cfg(feature = "once_cell")]
pub static RECORDS: once_cell::sync::Lazy<Records> = once_cell::sync::Lazy::new(|| {
    let csv = include_str!("../data/IP2LOCATION-ISO3166-2.CSV");
    Records::from_csv(csv.as_bytes()).unwrap()
});

#[cfg(feature = "once_cell")]
pub static RECORDS_CODE_MAP: once_cell::sync::Lazy<std::collections::HashMap<Box<str>, Record>> =
    once_cell::sync::Lazy::new(|| {
        RECORDS
            .iter()
            .cloned()
            .map(|x| (x.code.to_owned(), x))
            .collect()
    });

//
#[derive(Debug, Clone)]
pub struct Records(pub Vec<Record>);

impl Deref for Records {
    type Target = Vec<Record>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

//
impl Records {
    pub fn from_csv<R: Read>(rdr: R) -> Result<Self, RecordsFromCsvError> {
        let mut rdr = CsvReader::from_reader(rdr);
        let iter = rdr.records();

        let mut inner = vec![];

        for record in iter {
            let record = record.map_err(RecordsFromCsvError::CsvParseFailed)?;
            let row: Record = record
                .deserialize(None)
                .map_err(RecordsFromCsvError::RecordDeFailed)?;
            inner.push(row);
        }

        Ok(Self(inner))
    }
}

//
#[derive(Debug)]
pub enum RecordsFromCsvError {
    CsvParseFailed(CsvError),
    RecordDeFailed(CsvError),
}

impl fmt::Display for RecordsFromCsvError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{:?}", self)
    }
}

impl std::error::Error for RecordsFromCsvError {}

==> ip2location-iso3166-2-0.3.0.crate: ip2location-iso3166-2-0.3.0/src/csv_format.rs <==
use core::{fmt, ops::Deref};
use std::io::Read;

use csv::{Error as CsvError, Reader, StringRecord};

use crate::record::Record;

//
#[cfg(feature = "once_cell")]
pub static RECORDS: once_cell::sync::Lazy<Records> = once_cell::sync::Lazy::new(|| {
    let csv = include_str!("../data/IP2LOCATION-ISO3166-2.CSV");
    Records::from_csv(csv.as_bytes()).unwrap()
});

#[cfg(feature = "once_cell")]
pub static RECORDS_CODE_MAP: once_cell::sync::Lazy<
    std::collections::HashMap<country_code::iso3166_2::SubdivisionCode, Record>,
> = once_cell::sync::Lazy::new(|| {
    RECORDS
        .iter()
        .cloned()
        .map(|x| (x.code.to_owned(), x))
        .collect()
});

//
#[derive(Debug, Clone)]
pub struct Records(pub Vec<Record>);

impl Deref for Records {
    type Target = Vec<Record>;

    fn deref(&self) -> &Self::Target {
        &self.0
    }
}

//
impl Records {
    pub fn from_csv<R: Read>(rdr: R) -> Result<Self, RecordsFromCsvError> {
        let mut rdr = Reader::from_reader(rdr);

        let mut inner = vec![];

        for record in rdr.records() {
            let record = record.map_err(RecordsFromCsvError::CsvParseFailed)?;

            /*
            LINE: "AI","Anguilla","-"
            */
            let record = if record.get(2) == Some("-") {
                StringRecord::from(vec![
                    &record[0],
                    &record[1],
                    format!("{}-", &record[0]).as_str(),
                ])
            } else {
                record
            };

            let row: Record = record
                .deserialize(None)
                .map_err(RecordsFromCsvError::RecordDeFailed)?;
            inner.push(row);
        }

        Ok(Self(inner))
    }
}

//
#[derive(Debug)]
pub enum RecordsFromCsvError {
    CsvParseFailed(CsvError),
    RecordDeFailed(CsvError),
}

impl fmt::Display for RecordsFromCsvError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{:?}", self)
    }
}

impl std::error::Error for RecordsFromCsvError {}
